/*
 * Hibernate, Relational Persistence for Idiomatic Java
 *
 * JBoss, Home of Professional Open Source
 * Copyright 2011 Red Hat Inc. and/or its affiliates and other contributors
 * as indicated by the @authors tag. All rights reserved.
 * See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU Lesser General Public License, v. 2.1.
 * This program is distributed in the hope that it will be useful, but WITHOUT A
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License,
 * v.2.1 along with this distribution; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */
package org.hibernate.ejb.test.ejb3configuration;

import javax.persistence.EntityManagerFactory;

import org.hibernate.SessionFactory;
import org.hibernate.SessionFactoryObserver;
import org.hibernate.ejb.AvailableSettings;
import org.hibernate.ejb.Ejb3Configuration;

/**
 * @author Emmanuel Bernard <emmanuel@hibernate.org>
 */
public class SessionFactoryObserverTest extends junit.framework.TestCase {
    public void testSessionFactoryObserverProperty() {
        Ejb3Configuration conf = new Ejb3Configuration();
        conf.setProperty( AvailableSettings.SESSION_FACTORY_OBSERVER, GoofySessionFactoryObserver.class.getName() );
        conf.addAnnotatedClass( Bell.class );
        try {
            final EntityManagerFactory entityManagerFactory = conf.buildEntityManagerFactory();
            entityManagerFactory.close();
            fail("GoofyException should have been thrown");
        }
        catch ( GoofyException e ) {
            //success
        }
    }

    public static class GoofySessionFactoryObserver implements SessionFactoryObserver {

        public void sessionFactoryCreated(SessionFactory factory) {
        }

        public void sessionFactoryClosed(SessionFactory factory) {
            throw new GoofyException();
        }
    }

    public static class GoofyException extends RuntimeException {

    }
}